# Copyright 2023-2024 Vlad Krupinskii <mrvladus@yandex.ru>
# SPDX-License-Identifier: MIT

import datetime
from typing import Callable
from errands.lib.functions import get_children
from gi.repository import Gtk, Adw, GObject, GLib  # type:ignore


class Box(Gtk.Box):
    """
    Gtk.Box with multiple children provided in list.
    """

    def __init__(self, children: list[Gtk.Widget] = [], **kwargs):
        super().__init__(**kwargs)
        for child in children:
            self.append(child)

    def for_each(self, callback: Callable):
        """Call function for each child as argument"""
        for child in get_children(self):
            callback(child)


class Button(Gtk.Button):
    """
    Gtk.Button which takes arguments:
    - label
    - icon
    - callback for "clicked" signal
    - shortcut.
    """

    def __init__(
        self,
        label: str = None,
        icon_name: str = None,
        on_click: Callable = None,
        shortcut: str = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.label: str = label
        content = Adw.ButtonContent()
        if icon_name:
            content.set_icon_name(icon_name)
        if label:
            content.set_label(label)
        if on_click:
            self.connect("clicked", on_click)
        if icon_name:
            self.set_child(content)
        else:
            self.set_label(label)
        if shortcut:
            ctrl = Gtk.ShortcutController(scope=1)
            ctrl.add_shortcut(
                Gtk.Shortcut(
                    trigger=Gtk.ShortcutTrigger.parse_string(shortcut),
                    action=Gtk.ShortcutAction.parse_string("activate"),
                )
            )
            self.add_controller(ctrl)


class DateTime(Gtk.Box):
    """
    Date and time picker widget.
    Datetime returned in yyyymmddThhmmss format.
    """

    datetime: str = ""
    changed = GObject.Signal()
    lock_signals: bool = True

    def __init__(self):
        super().__init__()
        self._build_ui()

    def _build_ui(self):
        self.set_orientation(Gtk.Orientation.VERTICAL)
        # Hour button
        self.hour = Gtk.SpinButton(
            adjustment=Gtk.Adjustment(lower=0, upper=23, step_increment=1)
        )
        self.hour.connect("value-changed", self._on_date_time_changed)
        # Minute button
        self.minutes = Gtk.SpinButton(
            adjustment=Gtk.Adjustment(lower=0, upper=59, step_increment=1)
        )
        self.minutes.connect("value-changed", self._on_date_time_changed)
        # Time box
        self.append(
            Box(
                children=[
                    Box(
                        children=[
                            self.hour,
                            Gtk.Label(label=":"),
                            self.minutes,
                        ],
                        css_classes=["toolbar"],
                        halign="center",
                    ),
                    Box(
                        children=[
                            Button(
                                "09:00",
                                "errands-daytime-morning-symbolic",
                                on_click=self._on_time_preset_clicked,
                            ),
                            Button(
                                "13:00",
                                "errands-theme-light-symbolic",
                                on_click=self._on_time_preset_clicked,
                            ),
                        ],
                        css_classes=["toolbar"],
                        homogeneous=True,
                    ),
                    Box(
                        children=[
                            Button(
                                "17:00",
                                "errands-daytime-sunset-symbolic",
                                on_click=self._on_time_preset_clicked,
                            ),
                            Button(
                                "20:00",
                                "errands-theme-dark-symbolic",
                                on_click=self._on_time_preset_clicked,
                            ),
                        ],
                        css_classes=["toolbar"],
                        homogeneous=True,
                    ),
                ],
                orientation="vertical",
            )
        )
        # Calendar
        self.calendar = Gtk.Calendar()
        self.calendar.connect("day-selected", self._on_date_time_changed)
        self.append(self.calendar)
        # Buttons box
        self.append(
            Box(
                children=[
                    Box(
                        children=[
                            Button(
                                label=_("Today"),
                                on_click=self._on_today_btn_clicked,
                            ),
                            Button(
                                label=_("Tomorrow"),
                                on_click=self._on_tomorrow_btn_clicked,
                            ),
                        ],
                        css_classes=["toolbar"],
                        homogeneous=True,
                    ),
                    Box(
                        children=[
                            Button(
                                label=_("Now"),
                                on_click=self._on_now_btn_clicked,
                                hexpand=True,
                            ),
                            Button(
                                label=_("Clear"),
                                on_click=self._on_clear_btn_clicked,
                                hexpand=True,
                            ),
                        ],
                        css_classes=["toolbar"],
                        homogeneous=True,
                    ),
                ],
                orientation="vertical",
            )
        )

    def _on_date_time_changed(self, *_args):
        # Get hour
        hour: str = str(self.hour.get_value_as_int())
        hour: str = f"0{hour}" if len(hour) == 1 else hour
        # Get min
        min: str = str(self.minutes.get_value_as_int())
        min: str = f"0{min}" if len(min) == 1 else min
        # Get date
        date: str = self.calendar.get_date().format("%Y%m%d")
        # Set date
        self.datetime = f"{date}T{hour}{min}00"
        if not self.lock_signals:
            self.emit("changed")

    def _on_time_preset_clicked(self, btn: Button):
        hour, min = btn.label.split(":")
        self.hour.set_value(int(hour))
        self.minutes.set_value(int(min))

    def _on_now_btn_clicked(self, _btn):
        self.set_datetime(datetime.datetime.now().strftime("%Y%m%dT%H%M00"))
        self.emit("changed")

    def _on_today_btn_clicked(self, _btn):
        self.set_datetime(datetime.datetime.now().strftime("%Y%m%dT000000"))
        self.emit("changed")

    def _on_tomorrow_btn_clicked(self, _btn):
        self.set_datetime(
            (datetime.datetime.now() + datetime.timedelta(1)).strftime("%Y%m%dT000000")
        )
        self.emit("changed")

    def _on_clear_btn_clicked(self, _btn):
        self.datetime = ""
        self.emit("changed")

    def get_datetime(self) -> str:
        return self.datetime

    def get_human_datetime(self) -> str:
        if self.datetime:
            out: str = (
                f"{self.datetime[9:11]}:{self.datetime[11:13]}, {self.calendar.get_date().format('%d %B, %Y')}"
            )
        else:
            out: str = _("Not Set")
        return out

    def get_datetime_as_int(self) -> int:
        return int(f"{self.datetime[:8]}{self.datetime[9:]}") if self.datetime else 0

    def set_datetime(self, dt: str):
        self.lock_signals = True
        if dt:
            dt = datetime.datetime.fromisoformat(dt).strftime("%Y%m%dT%H%M00")
            self.hour.set_value(int(dt[9:11]))
            self.minutes.set_value(int(dt[11:13]))
            self.calendar.select_day(
                GLib.DateTime.new_local(
                    int(dt[:4]), int(dt[4:6]), int(dt[6:8]), 0, 0, 0
                )
            )
        else:
            self.hour.set_value(0)
            self.minutes.set_value(0)
            self.calendar.select_day(GLib.DateTime.new_now_local())
        # Set datetime
        self.datetime = dt
        self.lock_signals = False



# Copyright 2023-2024 Vlad Krupinskii <mrvladus@yandex.ru>
# SPDX-License-Identifier: MIT

from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from errands.widgets.window import Window

from errands.lib.goa import get_goa_credentials
from gi.repository import Adw, Gtk  # type:ignore
from errands.lib.sync.sync import Sync
from errands.lib.gsettings import GSettings


class PreferencesWindow(Adw.PreferencesWindow):
    selected_provider: int = 0

    def __init__(self, win: Window) -> None:
        super().__init__()
        self.window: Window = win
        self._build_ui()
        self._setup_sync()

    def _build_ui(self) -> None:
        self.set_transient_for(self.window)
        self.set_search_enabled(False)

        # Theme group
        theme_group: Adw.PreferencesGroup = Adw.PreferencesGroup(
            title=_("Application Theme"),
        )
        # System theme
        self.theme_system_btn: Gtk.CheckButton = Gtk.CheckButton(
            active=GSettings.get("theme") == 0
        )
        self.theme_system_btn.connect("toggled", self.on_theme_change, 0)
        theme_system_row: Adw.ActionRow = Adw.ActionRow(
            title=_("System"),
            icon_name="errands-theme-system-symbolic",
        )
        theme_system_row.add_suffix(self.theme_system_btn)
        theme_system_row.set_activatable_widget(self.theme_system_btn)
        theme_group.add(theme_system_row)
        # Light theme
        self.theme_light_btn: Gtk.CheckButton = Gtk.CheckButton(
            group=self.theme_system_btn, active=GSettings.get("theme") == 1
        )
        self.theme_light_btn.connect("toggled", self.on_theme_change, 1)
        theme_light_row: Adw.ActionRow = Adw.ActionRow(
            title=_("Light"),
            icon_name="errands-theme-light-symbolic",
        )
        theme_light_row.add_suffix(self.theme_light_btn)
        theme_light_row.set_activatable_widget(self.theme_light_btn)
        theme_group.add(theme_light_row)
        # Dark theme
        self.theme_dark_btn = Gtk.CheckButton(
            group=self.theme_system_btn, active=GSettings.get("theme") == 4
        )
        self.theme_dark_btn.connect("toggled", self.on_theme_change, 4)
        theme_dark_row: Adw.ActionRow = Adw.ActionRow(
            title=_("Dark"),
            icon_name="errands-theme-dark-symbolic",
        )
        theme_dark_row.add_suffix(self.theme_dark_btn)
        theme_dark_row.set_activatable_widget(self.theme_dark_btn)
        theme_group.add(theme_dark_row)

        # Task lists group
        # task_list_group = Adw.PreferencesGroup(title=_("Task Lists"))
        # add_tasks_position = Adw.ComboRow(
        #     title=_("Add new Tasks"),
        #     model=Gtk.StringList.new([_("At the Top"), _("At the Bottom")]),
        #     icon_name="errands-add-symbolic",
        # )
        # task_list_group.add(add_tasks_position)

        # Tasks group
        tasks_group = Adw.PreferencesGroup(title=_("Tasks"))
        # Primary action
        task_primary_action = Adw.ComboRow(
            title=_("Click Action"),
            model=Gtk.StringList.new([_("Open Details Panel"), _("Show Sub-Tasks")]),
            icon_name="errands-click-symbolic",
        )
        task_primary_action.set_selected(
            int(GSettings.get("primary-action-show-sub-tasks"))
        )
        task_primary_action.connect(
            "notify::selected",
            lambda row, *_: GSettings.set(
                "primary-action-show-sub-tasks", "b", bool(row.get_selected())
            ),
        )
        tasks_group.add(task_primary_action)
        # Toggle size
        task_big_toggle = Adw.ComboRow(
            title=_("Complete Button Size"),
            model=Gtk.StringList.new([_("Small"), _("Big")]),
            icon_name="errands-check-toggle-symbolic",
        )
        task_big_toggle.set_selected(int(GSettings.get("task-big-toggle")))
        task_big_toggle.connect(
            "notify::selected",
            lambda row, *_: GSettings.set(
                "task-big-toggle", "b", bool(row.get_selected())
            ),
        )
        tasks_group.add(task_big_toggle)
        # Progress bar
        task_progress_bar = Adw.SwitchRow(
            title=_("Progress Bar"), icon_name="errands-progressbar-symbolic"
        )
        GSettings.bind("task-show-progressbar", task_progress_bar, "active")
        tasks_group.add(task_progress_bar)
        # Toolbar
        task_toolbar = Adw.SwitchRow(
            title=_("Tool Bar"), icon_name="errands-toolbar-symbolic"
        )
        GSettings.bind("task-show-toolbar", task_toolbar, "active")
        # tasks_group.add(task_toolbar)

        # Sync group
        sync_group = Adw.PreferencesGroup(
            title=_("Sync"),
        )
        # Provider
        model = Gtk.StringList.new([_("Disabled"), "Nextcloud", "CalDAV"])
        self.sync_providers = Adw.ComboRow(
            title=_("Sync Provider"),
            model=model,
            icon_name="errands-sync-symbolic",
        )
        GSettings.bind("sync-provider", self.sync_providers, "selected")
        self.sync_providers.connect("notify::selected", lambda *_: self._setup_sync())
        sync_group.add(self.sync_providers)
        # URL
        self.sync_url = Adw.EntryRow(
            title=_("Server URL"),
        )
        GSettings.bind("sync-url", self.sync_url, "text")
        sync_group.add(self.sync_url)
        # Username
        self.sync_username = Adw.EntryRow(
            title=_("Username"),
        )
        GSettings.bind("sync-username", self.sync_username, "text")
        sync_group.add(self.sync_username)
        # Password
        self.sync_password = Adw.PasswordEntryRow(
            title=_("Password"),
        )
        self.sync_password.connect("changed", self.on_sync_pass_changed)
        sync_group.add(self.sync_password)
        # Test connection
        test_btn = Gtk.Button(
            label=_("Test"),
            valign="center",
        )
        test_btn.connect("clicked", self.on_test_connection_btn_clicked)
        self.test_connection_row = Adw.ActionRow(
            title=_("Test Connection"),
        )
        self.test_connection_row.add_suffix(test_btn)
        self.test_connection_row.set_activatable_widget(test_btn)
        sync_group.add(self.test_connection_row)

        # Details group
        details_group = Adw.PreferencesGroup(
            title=_("Details Panel"),
        )
        details_position = Adw.ComboRow(
            title=_("Position"),
            model=Gtk.StringList.new([_("Left"), _("Right")]),
            icon_name="errands-sidebar-left-symbolic",
        )
        details_position.set_selected(int(GSettings.get("right-sidebar")))
        details_position.connect("notify::selected", self._on_details_position_changed)
        details_group.add(details_position)

        # Appearance Page
        appearance_page = Adw.PreferencesPage(
            title=_("Appearance"), icon_name="errands-appearance-symbolic"
        )
        appearance_page.add(theme_group)
        appearance_page.add(tasks_group)
        appearance_page.add(details_group)
        # page.add(task_list_group)
        self.add(appearance_page)

        # Sync Page
        sync_page = Adw.PreferencesPage(
            title=_("Sync"), icon_name="errands-sync-symbolic"
        )
        sync_page.add(sync_group)
        self.add(sync_page)

    def _setup_sync(self) -> None:
        selected: int = self.sync_providers.props.selected
        self.sync_url.set_visible(0 < selected < 3)
        self.sync_username.set_visible(0 < selected < 3)
        self.sync_password.set_visible(0 < selected < 3)
        self.test_connection_row.set_visible(selected > 0)

        if self.sync_password.props.visible:
            account: str = self.sync_providers.props.selected_item.props.string
            password: str = GSettings.get_secret(account)
            with self.sync_password.freeze_notify():
                self.sync_password.props.text = password if password else ""

        # Fill out forms from Gnome Online Accounts if needed
        acc_name: str = self.sync_providers.props.selected_item.props.string
        data: dict[str, str] | None = get_goa_credentials(acc_name)
        if data:
            if not GSettings.get("sync-url"):
                self.sync_url.set_text(data["url"])
            if not GSettings.get("sync-username"):
                self.sync_username.set_text(data["username"])
            if not GSettings.get_secret(acc_name):
                self.sync_password.set_text(data["password"])

    def on_sync_pass_changed(self, _entry) -> None:
        if 0 < self.sync_providers.props.selected < 3:
            account = self.sync_providers.props.selected_item.props.string
            GSettings.set_secret(account, self.sync_password.props.text)

    def on_test_connection_btn_clicked(self, _btn) -> None:
        res: bool = Sync.test_connection()
        msg: str = _("Connected") if res else _("Can't connect")
        toast: Adw.Toast = Adw.Toast(title=msg, timeout=2)
        self.add_toast(toast)

    def on_theme_change(self, btn: Gtk.Button, theme: int) -> None:
        Adw.StyleManager.get_default().set_color_scheme(theme)
        GSettings.set("theme", "i", theme)

    def _on_details_position_changed(self, row: Adw.ComboRow, *_) -> None:
        for list in self.window.sidebar.task_lists._get_task_lists():
            list.split_view.set_sidebar_position(row.get_selected())
        GSettings.set("right-sidebar", "b", bool(row.get_selected())),
